@@ -20,7 +20,7 @@ use tor_config::MutCfg;
 use tor_dirmgr::bridgedesc::BridgeDescMgr;
 use tor_dirmgr::{DirMgrStore, Timeliness};
 use tor_error::{error_report, internal, Bug};
-use tor_guardmgr::GuardMgr;
+use tor_guardmgr::{GuardMgr, RetireCircuits};
 use tor_netdir::{params::NetParameters, NetDirProvider};
 #[cfg(feature = "onion-service-service")]
 use tor_persist::state_dir::StateDirectory;
@@ -52,6 +52,8 @@ use std::result::Result as StdResult;
 use std::sync::{Arc, Mutex};
 
 use crate::err::ErrorDetail;
+#[cfg(feature = "rpc")]
+use crate::rpc::ClientConnectionTarget;
 use crate::{status, util, TorClientBuilder};
 #[cfg(feature = "geoip")]
 use tor_geoip::CountryCode;
@@ -70,7 +72,13 @@ use tracing::{debug, info};
 // implicit Arcs inside them! maybe it's time to replace much of the insides of
 // this with an Arc<TorClientInner>?
 #[derive(Clone)]
-#[cfg_attr(feature = "rpc", derive(Deftly), derive_deftly(Object))]
+#[cfg_attr(
+    feature = "rpc",
+    derive(Deftly),
+    derive_deftly(Object),
+    deftly(rpc(expose_outside_of_session)),
+    deftly(rpc(downcastable_to = "ClientConnectionTarget"))
+)]
 pub struct TorClient<R: Runtime> {
     /// Asynchronous runtime object.
     runtime: R,
@@ -346,6 +354,13 @@ impl StreamPrefs {
         self
     }
 
+    /// Return true if this stream has been configured as "optimistic".
+    ///
+    /// See [`StreamPrefs::optimistic`] for more info.
+    pub fn is_optimistic(&self) -> bool {
+        self.optimistic_stream
+    }
+
     /// Indicate whether connection to a hidden service (`.onion` service) should be allowed
     ///
     /// If `Explicit(false)`, attempts to connect to Onion Services will be forced to fail with
@@ -889,17 +904,15 @@ impl<R: Runtime> TorClient<R> {
             how.cannot_change("storage.state_dir").map_err(wrap_err)?;
         }
 
-        self.circmgr
+        let retire_circuits = self
+            .circmgr
             .reconfigure(new_config, how)
             .map_err(wrap_err)?;
 
-        #[cfg(all(
-            feature = "vanguards",
-            any(feature = "onion-service-client", feature = "onion-service-service")
-        ))]
-        self.hs_circ_pool
-            .reconfigure(new_config, how)
-            .map_err(wrap_err)?;
+        #[cfg(any(feature = "onion-service-client", feature = "onion-service-service"))]
+        if retire_circuits != RetireCircuits::None {
+            self.hs_circ_pool.retire_all_circuits().map_err(wrap_err)?;
+        }
 
         self.dirmgr.reconfigure(&dir_cfg, how).map_err(wrap_err)?;
 
