@@ -223,7 +223,7 @@ impl Connection {
     {
         let write = Box::pin(asynchronous_codec::FramedWrite::new(
             output,
-            crate::streams::JsonLinesEncoder::<BoxedResponse>::default(),
+            crate::codecs::JsonLinesEncoder::<BoxedResponse>::default(),
         ));
 
         let read = Box::pin(
@@ -407,18 +407,19 @@ impl Connection {
         self: &Arc<Self>,
         tx_updates: rpc::dispatch::BoxedUpdateSink,
         obj: rpc::ObjectId,
-        method: Box<dyn rpc::DynMethod>,
+        method: Box<dyn rpc::DeserMethod>,
     ) -> Result<Box<dyn erased_serde::Serialize + Send + 'static>, rpc::RpcError> {
         let obj = self.lookup_object(&obj)?;
 
         let context: Box<dyn rpc::Context> = Box::new(RequestContext {
             conn: Arc::clone(self),
         });
-        let invoke_future = self
-            .dispatch_table
-            .read()
-            .expect("lock poisoned")
-            .invoke(obj, method, context, tx_updates)?;
+        let invoke_future = self.dispatch_table.read().expect("lock poisoned").invoke(
+            obj,
+            method.upcast_box(),
+            context,
+            tx_updates,
+        )?;
 
         // Note that we drop the read lock before we await this future!
         invoke_future.await
@@ -531,6 +532,10 @@ impl rpc::Context for RequestContext {
             Err(rpc::LookupError::NoObject(id.clone()))
         }
     }
+
+    fn dispatch_table(&self) -> &Arc<std::sync::RwLock<rpc::DispatchTable>> {
+        &self.conn.dispatch_table
+    }
 }
 
 /// An error given when an RPC request is cancelled.
