@@ -13,8 +13,9 @@ use std::{
 use crate::{timeouts, CircMgr, Error, Result};
 use futures::{task::SpawnExt, StreamExt, TryFutureExt};
 use once_cell::sync::OnceCell;
-use tor_error::debug_report;
 use tor_error::{bad_api_usage, internal};
+use tor_error::{debug_report, Bug};
+use tor_guardmgr::VanguardMode;
 use tor_linkspec::{CircTarget, HasRelayIds as _, OwnedCircTarget, RelayIdSet};
 use tor_netdir::{NetDir, NetDirProvider, Relay};
 use tor_proto::circuit::{self, CircParameters, ClientCirc};
@@ -58,10 +59,10 @@ impl HsCircKind {
     /// Return the [`HsCircStubKind`] needed to build this type of circuit.
     fn stub_kind(&self) -> HsCircStubKind {
         match self {
-            HsCircKind::ClientRend | HsCircKind::SvcIntro => HsCircStubKind::Stub,
+            HsCircKind::ClientRend | HsCircKind::SvcIntro => HsCircStubKind::Short,
             HsCircKind::SvcHsDir => {
-                // TODO HS-VANGUARDS: we might want this to be STUB+
-                HsCircStubKind::Stub
+                // TODO: we might want this to be EXTENDED
+                HsCircStubKind::Short
             }
             HsCircKind::SvcRend | HsCircKind::ClientHsDir | HsCircKind::ClientIntro => {
                 HsCircStubKind::Extended
@@ -78,7 +79,7 @@ impl HsCircKind {
 pub(crate) struct HsCircStub {
     /// The circuit.
     pub(crate) circ: Arc<ClientCirc>,
-    /// Whether the circuit is STUB or STUB+.
+    /// Whether the circuit is SHORT or EXTENDED
     pub(crate) kind: HsCircStubKind,
 }
 
@@ -109,14 +110,14 @@ impl HsCircStub {
     /// or can be extended to become that kind.
     ///
     /// Returns `true` if this `HsCircStub`'s kind is equal to `other`,
-    /// or if its kind is [`Stub`](HsCircStubKind::Stub)
+    /// or if its kind is [`Short`](HsCircStubKind::Short)
     /// and `other` is [`Extended`](HsCircStubKind::Extended).
     pub(crate) fn can_become(&self, other: HsCircStubKind) -> bool {
         use HsCircStubKind::*;
 
         match (self.kind, other) {
-            (Stub, Stub) | (Extended, Extended) | (Stub, Extended) => true,
-            (Extended, Stub) => false,
+            (Short, Short) | (Extended, Extended) | (Short, Extended) => true,
+            (Extended, Short) => false,
         }
     }
 }
@@ -129,28 +130,34 @@ impl HsCircStub {
 ///
 ///   * with vanguards disabled:
 ///      ```text
-///         STUB  = G -> M -> M
-///         STUB+ = G -> M -> M
+///         SHORT    = G -> M -> M
+///         EXTENDED = G -> M -> M
 ///      ```
 ///
 ///   * with lite vanguards enabled:
 ///      ```text
-///         STUB  = G -> L2 -> M
-///         STUB+ = G -> L2 -> M
+///         SHORT    = G -> L2 -> M
+///         EXTENDED = G -> L2 -> M
 ///      ```
 ///
 ///   * with full vanguards enabled:
 ///      ```text
-///         STUB  = G -> L2 -> L3
-///         STUB+ = G -> L2 -> L3 -> M
+///         SHORT    = G -> L2 -> L3
+///         EXTENDED = G -> L2 -> L3 -> M
 ///      ```
 #[derive(Copy, Clone, Debug, PartialEq, derive_more::Display)]
 pub(crate) enum HsCircStubKind {
-    /// A stub circuit (STUB).
-    #[display(fmt = "STUB")]
-    Stub,
-    /// An extended stub circuit (STUB+).
-    #[display(fmt = "STUB+")]
+    /// A short stub circuit.
+    ///
+    /// Used for building circuits to a final hop that an adversary cannot easily control,
+    /// for example if the final hop is is randomly chosen by us.
+    #[display(fmt = "SHORT")]
+    Short,
+    /// An extended stub circuit.
+    ///
+    /// Used for building circuits to a final hop that an adversary can easily control,
+    /// for example if the final hop is not chosen by us.
+    #[display(fmt = "EXTENDED")]
     Extended,
 }
 
@@ -236,11 +243,16 @@ impl<R: Runtime> HsCircPool<R> {
         //   * the weighting rules for selecting rendezvous points are the same
         //     as those for selecting an arbitrary middle relay.
         let circ = self
-            .take_or_launch_stub_circuit::<OwnedCircTarget>(netdir, None, HsCircStubKind::Stub)
+            .take_or_launch_stub_circuit::<OwnedCircTarget>(netdir, None, HsCircStubKind::Short)
             .await?;
 
-        if self.vanguards_enabled() && circ.kind != HsCircStubKind::Stub {
-            return Err(internal!("wanted a STUB circuit, but got STUB+?!").into());
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+        if matches!(
+            self.vanguard_mode(),
+            VanguardMode::Full | VanguardMode::Lite
+        ) && circ.kind != HsCircStubKind::Short
+        {
+            return Err(internal!("wanted a SHORT circuit, but got EXTENDED?!").into());
         }
 
         let path = circ.path_ref();
@@ -296,7 +308,12 @@ impl<R: Runtime> HsCircPool<R> {
             .take_or_launch_stub_circuit(netdir, Some(&target), wanted_kind)
             .await?;
 
-        if self.vanguards_enabled() && circ.kind != wanted_kind {
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+        if matches!(
+            self.vanguard_mode(),
+            VanguardMode::Full | VanguardMode::Lite
+        ) && circ.kind != wanted_kind
+        {
             return Err(internal!(
                 "take_or_launch_stub_circuit() returned {:?}, but we need {wanted_kind:?}",
                 circ.kind
@@ -350,20 +367,17 @@ impl<R: Runtime> HsCircPool<R> {
         Ok(circ.circ)
     }
 
-    /// Try to change our configuration to `new_config`.
+    /// Retire the circuits in this pool.
     ///
-    /// Actual behavior will depend on the value of `how`.
-    pub fn reconfigure<CFG: HsCircPoolConfig>(
-        &self,
-        new_config: &CFG,
-        _how: tor_config::Reconfigure,
-    ) -> StdResult<(), tor_config::ReconfigureError> {
-        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+    /// This is used for handling vanguard configuration changes:
+    /// if the [`VanguardMode`] changes, we need to empty the pool and rebuild it,
+    /// because the old circuits are no longer suitable for use.
+    pub fn retire_all_circuits(&self) -> StdResult<(), tor_config::ReconfigureError> {
         self.inner
             .lock()
             .expect("poisoned lock")
             .pool
-            .reconfigure_vanguards(new_config.vanguard_config())?;
+            .retire_all_circuits()?;
 
         Ok(())
     }
@@ -387,6 +401,13 @@ impl<R: Runtime> HsCircPool<R> {
         // family info.
         T: CircTarget,
     {
+        let vanguard_mode = self.vanguard_mode();
+        trace!(
+            vanguards=%vanguard_mode,
+            kind=%kind,
+            "selecting HS circuit stub"
+        );
+
         // First, look for a circuit that is already built, if any is suitable.
 
         let target_exclusion = {
@@ -402,22 +423,30 @@ impl<R: Runtime> HsCircPool<R> {
 
         let found_usable_circ = {
             let mut inner = self.inner.lock().expect("lock poisoned");
-            let vanguards_enabled = inner.pool.vanguards_enabled();
 
             let restrictions = |circ: &HsCircStub| {
                 // If vanguards are enabled, we no longer apply same-family or same-subnet
                 // restrictions, and we allow the guard to appear as either of the last
                 // two hope of the circuit.
-                if vanguards_enabled {
-                    circ.can_become(kind) && circuit_still_useable(netdir, circ, |_relay| true)
-                } else {
-                    circuit_compatible_with_target(netdir, circ, &target_exclusion)
+                match vanguard_mode {
+                    #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+                    VanguardMode::Lite | VanguardMode::Full => {
+                        vanguards_circuit_compatible_with_target(netdir, circ, kind, avoid_target)
+                    }
+                    VanguardMode::Disabled => {
+                        circuit_compatible_with_target(netdir, circ, &target_exclusion)
+                    }
+                    _ => {
+                        warn!("unknown vanguard mode {vanguard_mode}");
+                        false
+                    }
                 }
             };
 
             let mut prefs = HsCircPrefs::default();
 
-            if vanguards_enabled {
+            #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+            if matches!(vanguard_mode, VanguardMode::Full | VanguardMode::Lite) {
                 prefs.preferred_stub_kind(kind);
             }
 
@@ -438,7 +467,11 @@ impl<R: Runtime> HsCircPool<R> {
         };
         // Return the circuit we found before, if any.
         if let Some(circuit) = found_usable_circ {
-            return self.maybe_extend_stub_circuit(netdir, circuit, kind).await;
+            let circuit = self
+                .maybe_extend_stub_circuit(netdir, circuit, avoid_target, kind)
+                .await?;
+            self.ensure_suitable_circuit(&circuit, avoid_target, kind)?;
+            return Ok(circuit);
         }
 
         // TODO: There is a possible optimization here. Instead of only waiting
@@ -452,28 +485,43 @@ impl<R: Runtime> HsCircPool<R> {
             .launch_hs_unmanaged(avoid_target, netdir, kind)
             .await?;
 
+        self.ensure_suitable_circuit(&circ, avoid_target, kind)?;
+
         Ok(HsCircStub { circ, kind })
     }
 
     /// Return a circuit of the specified `kind`, built from `circuit`.
-    async fn maybe_extend_stub_circuit(
+    async fn maybe_extend_stub_circuit<T>(
         &self,
         netdir: &NetDir,
         circuit: HsCircStub,
+        avoid_target: Option<&T>,
         kind: HsCircStubKind,
-    ) -> Result<HsCircStub> {
-        if !self.vanguards_enabled() {
-            return Ok(circuit);
+    ) -> Result<HsCircStub>
+    where
+        T: CircTarget,
+    {
+        match self.vanguard_mode() {
+            #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+            VanguardMode::Full => {
+                // SHORT circuit stubs need to be extended by one hop to become EXTENDED stubs
+                // if we're using full vanguards.
+            }
+            _ => {
+                let HsCircStub { circ, kind: _ } = circuit;
+
+                return Ok(HsCircStub { circ, kind });
+            }
         }
 
         match (circuit.kind, kind) {
-            (HsCircStubKind::Stub, HsCircStubKind::Extended) => {
-                debug!("Wanted STUB+ circuit, but got STUB; extending by 1 hop...");
+            (HsCircStubKind::Short, HsCircStubKind::Extended) => {
+                debug!("Wanted EXTENDED circuit, but got SHORT; extending by 1 hop...");
                 let params = CircParameters::default();
                 let usage = RelayUsage::middle_relay(Some(&RelayUsage::middle_relay(None)));
                 let circ_path = circuit.circ.path_ref();
 
-                // A STUB circuit is a 3-hop circuit.
+                // A SHORT circuit is a 3-hop circuit.
                 debug_assert_eq!(circ_path.hops().len(), 3);
 
                 // Like in VanguardHsPathBuilder::pick_path, we only want to exclude the L2 and L3
@@ -488,26 +536,36 @@ impl<R: Runtime> HsCircPool<R> {
                     exclude_ids.extend(hop.identities().map(|id| id.to_owned()));
                 }
 
-                let exclusion = RelayExclusion::exclude_identities(exclude_ids);
+                // We also must exclude the target
+                let target_exclusion = if let Some(target) = &avoid_target {
+                    RelayExclusion::exclude_identities(
+                        target.identities().map(|id| id.to_owned()).collect(),
+                    )
+                } else {
+                    RelayExclusion::no_relays_excluded()
+                };
+
+                let mut exclusion = RelayExclusion::exclude_identities(exclude_ids);
+                exclusion.extend(&target_exclusion);
                 let selector = RelaySelector::new(usage, exclusion);
                 let target = {
                     let mut rng = rand::thread_rng();
                     let (relay, info) = selector.select_relay(&mut rng, netdir);
                     relay.ok_or_else(|| Error::NoRelay {
-                        path_kind: "vanguard STUB+",
+                        path_kind: "vanguard EXTENDED",
                         role: "final hop",
                         problem: info.to_string(),
                     })?
                 };
 
-                // If full vanguards are enabled and the circuit we got is STUB,
-                // we need to extend it by another hop to make it STUB+ before returning it
+                // Since full vanguards are enabled and the circuit we got is SHORT,
+                // we need to extend it by another hop to make it EXTENDED before returning it
                 let circ = self.extend_circ(circuit, params, target).await?;
 
                 Ok(HsCircStub { circ, kind })
             }
-            (HsCircStubKind::Extended, HsCircStubKind::Stub) => {
-                Err(internal!("wanted a STUB circuit, but got STUB+?!").into())
+            (HsCircStubKind::Extended, HsCircStubKind::Short) => {
+                Err(internal!("wanted a SHORT circuit, but got EXTENDED?!").into())
             }
             _ => {
                 trace!("Wanted {kind} circuit, got {}", circuit.kind);
@@ -517,6 +575,93 @@ impl<R: Runtime> HsCircPool<R> {
         }
     }
 
+    /// Ensure `circ` is compatible with `target`, and has the correct length for its `kind`.
+    fn ensure_suitable_circuit<T>(
+        &self,
+        circ: &Arc<ClientCirc>,
+        target: Option<&T>,
+        kind: HsCircStubKind,
+    ) -> StdResult<(), Bug>
+    where
+        T: CircTarget,
+    {
+        Self::ensure_circuit_compatible_with_target(circ, target)?;
+        self.ensure_circuit_length_valid(circ, kind)?;
+
+        Ok(())
+    }
+
+    /// Ensure the specified circuit of type `kind` has the right length.
+    fn ensure_circuit_length_valid(
+        &self,
+        circ: &Arc<ClientCirc>,
+        kind: HsCircStubKind,
+    ) -> StdResult<(), Bug> {
+        let circ_path_len = circ.path_ref().n_hops();
+        let mode = self.vanguard_mode();
+
+        // TODO: the expected circuit lengths are hard-coded and duplicated in hspath.rs
+        // TODO(#1457): somehow unify the path length checks
+        let expected_len = match (mode, kind) {
+            #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+            (VanguardMode::Lite, _) => 3,
+            #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+            (VanguardMode::Full, HsCircStubKind::Short) => 3,
+            #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+            (VanguardMode::Full, HsCircStubKind::Extended) => 4,
+            (VanguardMode::Disabled, _) => 3,
+            (_, _) => {
+                return Err(internal!("Unsupported vanguard mode {mode}"));
+            }
+        };
+
+        if circ_path_len != expected_len {
+            return Err(internal!(
+                "invalid path length for {} {mode}-vanguard circuit (expected {} hops, got {})",
+                kind,
+                expected_len,
+                circ_path_len
+            ));
+        }
+
+        Ok(())
+    }
+
+    /// Ensure `circ` is compatible with `target`.
+    ///
+    /// Returns an error if either of the last 2 hops of the circuit are the same as `target`,
+    /// because:
+    ///   * a relay won't let you extend the circuit to itself
+    ///   * relays won't let you extend the circuit to their previous hop
+    fn ensure_circuit_compatible_with_target<T>(
+        circ: &Arc<ClientCirc>,
+        target: Option<&T>,
+    ) -> StdResult<(), Bug>
+    where
+        T: CircTarget,
+    {
+        if let Some(target) = target {
+            let take_n = 2;
+            if let Some(hop) = circ
+                .path_ref()
+                .hops()
+                .iter()
+                .rev()
+                .take(take_n)
+                .flat_map(|hop| hop.as_chan_target())
+                .find(|hop| hop.has_any_relay_id_from(target))
+            {
+                return Err(internal!(
+                    "invalid path: circuit target {} appears as one of the last 2 hops (matches hop {})",
+                    target.display_relay_ids(),
+                    hop.display_relay_ids()
+                ));
+            }
+        }
+
+        Ok(())
+    }
+
     /// Internal: Remove every closed circuit from this pool.
     fn remove_closed(&self) {
         let mut inner = self.inner.lock().expect("lock poisoned");
@@ -532,14 +677,18 @@ impl<R: Runtime> HsCircPool<R> {
             .retain(|circ| circuit_still_useable(netdir, circ, |_relay| true));
     }
 
-    /// Returns `true` if vanguards are enabled.
-    fn vanguards_enabled(&self) -> bool {
+    /// Returns the current [`VanguardMode`].
+    fn vanguard_mode(&self) -> VanguardMode {
         cfg_if::cfg_if! {
             if #[cfg(all(feature = "vanguards", feature = "hs-common"))] {
-                let inner = self.inner.lock().expect("lock poisoned");
-                inner.pool.vanguards_enabled()
+                self
+                    .circmgr
+                    .mgr
+                    .peek_builder()
+                    .vanguardmgr()
+                    .mode()
             } else {
-                false
+                VanguardMode::Disabled
             }
         }
     }
@@ -588,6 +737,41 @@ fn circuit_compatible_with_target(
     })
 }
 
+/// Return true if we can extend a pre-built vanguards circuit `circ` to `target`.
+///
+/// We require that the circuit is open, that it can become the specified
+/// kind of [`HsCircStub`], that every hop in the circuit is listed in `netdir`,
+/// and that the last two hops are different from the specified target.
+fn vanguards_circuit_compatible_with_target<T>(
+    netdir: &NetDir,
+    circ: &HsCircStub,
+    kind: HsCircStubKind,
+    avoid_target: Option<&T>,
+) -> bool
+where
+    T: CircTarget,
+{
+    if let Some(target) = avoid_target {
+        let circ_path = circ.circ.path_ref();
+        // The last 2 hops of the circuit must be different from the circuit target, because:
+        //   * a relay won't let you extend the circuit to itself
+        //   * relays won't let you extend the circuit to their previous hop
+        let take_n = 2;
+        if circ_path
+            .hops()
+            .iter()
+            .rev()
+            .take(take_n)
+            .flat_map(|hop| hop.as_chan_target())
+            .any(|hop| hop.has_any_relay_id_from(target))
+        {
+            return false;
+        }
+    }
+
+    circ.can_become(kind) && circuit_still_useable(netdir, circ, |_relay| true)
+}
+
 /// Return true if we can still use a given pre-build circuit.
 ///
 /// We require that the circuit is open, that every hop  in the circuit is
@@ -648,7 +832,7 @@ async fn launch_hs_circuits_as_needed<R: Runtime>(
 
         if n_to_launch > 0 {
             debug!(
-                "launching {} STUB and {} STUB+ circuits",
+                "launching {} SHORT and {} EXTENDED circuits",
                 circs_to_launch.stub(),
                 circs_to_launch.ext_stub()
             );
@@ -733,3 +917,88 @@ async fn remove_unusable_circuits<R: Runtime>(
         }
     }
 }
+
+#[cfg(test)]
+mod test {
+    // @@ begin test lint list maintained by maint/add_warning @@
+    #![allow(clippy::bool_assert_comparison)]
+    #![allow(clippy::clone_on_copy)]
+    #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
+    #![allow(clippy::print_stderr)]
+    #![allow(clippy::print_stdout)]
+    #![allow(clippy::single_char_pattern)]
+    #![allow(clippy::unwrap_used)]
+    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::useless_vec)]
+    #![allow(clippy::needless_pass_by_value)]
+    //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
+    #![allow(clippy::cognitive_complexity)]
+
+    use tor_config::ExplicitOrAuto;
+    #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+    use tor_guardmgr::VanguardConfigBuilder;
+    use tor_guardmgr::VanguardMode;
+    use tor_rtmock::MockRuntime;
+
+    use super::*;
+    use crate::{CircMgr, TestConfig};
+
+    /// Create a `CircMgr` with an underlying `VanguardMgr` that runs in the specified `mode`.
+    fn circmgr_with_vanguards<R: Runtime>(runtime: R, mode: VanguardMode) -> Arc<CircMgr<R>> {
+        let chanmgr = tor_chanmgr::ChanMgr::new(
+            runtime.clone(),
+            &Default::default(),
+            tor_chanmgr::Dormancy::Dormant,
+            &Default::default(),
+        );
+        let guardmgr = tor_guardmgr::GuardMgr::new(
+            runtime.clone(),
+            tor_persist::TestingStateMgr::new(),
+            &tor_guardmgr::TestConfig::default(),
+        )
+        .unwrap();
+
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+        let vanguard_config = VanguardConfigBuilder::default()
+            .mode(ExplicitOrAuto::Explicit(mode))
+            .build()
+            .unwrap();
+
+        let config = TestConfig {
+            #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+            vanguard_config,
+            ..Default::default()
+        };
+
+        CircMgr::new(
+            &config,
+            tor_persist::TestingStateMgr::new(),
+            &runtime,
+            Arc::new(chanmgr),
+            guardmgr,
+        )
+        .unwrap()
+    }
+
+    #[test]
+    fn pool_with_vanguards_disabled() {
+        MockRuntime::test_with_various(|runtime| async move {
+            let circmgr = circmgr_with_vanguards(runtime, VanguardMode::Disabled);
+            let circpool = HsCircPool::new(&circmgr);
+            assert!(circpool.vanguard_mode() == VanguardMode::Disabled);
+        });
+    }
+
+    #[test]
+    #[cfg(all(feature = "vanguards", feature = "hs-common"))]
+    fn pool_with_vanguards_enabled() {
+        MockRuntime::test_with_various(|runtime| async move {
+            for mode in [VanguardMode::Lite, VanguardMode::Full] {
+                let circmgr = circmgr_with_vanguards(runtime.clone(), mode);
+                let circpool = HsCircPool::new(&circmgr);
+                assert!(circpool.vanguard_mode() == mode);
+            }
+        });
+    }
+}
