@@ -5,13 +5,14 @@
 //! [vanguards spec]: https://spec.torproject.org/vanguards-spec/index.html.
 
 pub mod config;
+mod err;
 mod set;
 
 use std::sync::{Arc, RwLock, Weak};
 use std::time::{Duration, SystemTime};
 
 use futures::stream::BoxStream;
-use futures::task::{SpawnError, SpawnExt as _};
+use futures::task::SpawnExt as _;
 use futures::{future, FutureExt as _};
 use futures::{select_biased, StreamExt as _};
 use postage::stream::Stream as _;
@@ -20,7 +21,7 @@ use rand::RngCore;
 
 use tor_async_utils::PostageWatchSenderExt as _;
 use tor_config::ReconfigureError;
-use tor_error::{error_report, internal, into_internal, ErrorKind, HasKind};
+use tor_error::{error_report, internal, into_internal};
 use tor_netdir::{DirEvent, NetDir, NetDirProvider, Timeliness};
 use tor_persist::{DynStorageHandle, StateMgr};
 use tor_relay_selection::RelayExclusion;
@@ -31,7 +32,9 @@ use crate::{RetireCircuits, VanguardMode};
 
 use set::VanguardSets;
 
-pub use config::{VanguardConfig, VanguardConfigBuilder, VanguardParams};
+use crate::VanguardConfig;
+pub use config::VanguardParams;
+pub use err::VanguardMgrError;
 pub use set::Vanguard;
 
 /// The key used for storing the vanguard sets to persistent storage using `StateMgr`.
@@ -113,49 +116,10 @@ enum ShutdownStatus {
     Terminate,
 }
 
-/// An error coming from the vanguards subsystem.
-#[derive(Clone, Debug, thiserror::Error)]
-#[non_exhaustive]
-pub enum VanguardMgrError {
-    /// Could not find a suitable relay to use for the specifier layer.
-    #[error("No suitable relays")]
-    NoSuitableRelay(Layer),
-
-    /// Could not get timely network directory.
-    #[error("Unable to get timely network directory")]
-    NetDir(#[from] tor_netdir::Error),
-
-    /// Failed to access persistent storage.
-    #[error("Failed to access persistent vanguard state")]
-    State(#[from] tor_persist::Error),
-
-    /// Could not spawn a task.
-    #[error("Unable to spawn a task")]
-    Spawn(#[source] Arc<SpawnError>),
-
-    /// An internal error occurred.
-    #[error("Internal error")]
-    Bug(#[from] tor_error::Bug),
-}
-
-impl HasKind for VanguardMgrError {
-    fn kind(&self) -> ErrorKind {
-        match self {
-            // TODO HS-VANGUARDS: this is not right
-            VanguardMgrError::NoSuitableRelay(_) => ErrorKind::Other,
-            VanguardMgrError::NetDir(e) => e.kind(),
-            VanguardMgrError::State(e) => e.kind(),
-            VanguardMgrError::Spawn(e) => e.kind(),
-            VanguardMgrError::Bug(e) => e.kind(),
-        }
-    }
-}
-
 impl<R: Runtime> VanguardMgr<R> {
     /// Create a new `VanguardMgr`.
     ///
     /// The `state_mgr` handle is used for persisting the "vanguards-full" guard pools to disk.
-    #[allow(clippy::needless_pass_by_value)] // TODO HS-VANGUARDS
     pub fn new<S>(
         config: &VanguardConfig,
         runtime: R,
@@ -165,7 +129,6 @@ impl<R: Runtime> VanguardMgr<R> {
     where
         S: StateMgr + Send + Sync + 'static,
     {
-        let VanguardConfig { mode } = config;
         // Note: we start out with default vanguard params, but we adjust them
         // as soon as we obtain a NetDir (see Self::run_once()).
         let params = VanguardParams::default();
@@ -192,7 +155,7 @@ impl<R: Runtime> VanguardMgr<R> {
         let (config_tx, _config_rx) = watch::channel();
         let inner = Inner {
             params,
-            mode: *mode,
+            mode: config.mode(),
             vanguard_sets,
             has_onion_svc,
             config_tx,
@@ -246,8 +209,9 @@ impl<R: Runtime> VanguardMgr<R> {
         // but not decessarily downgrade to lite if we stop.
         // See <https://gitlab.torproject.org/tpo/core/arti/-/merge_requests/2083#note_3018173>
         let mut inner = self.inner.write().expect("poisoned lock");
-        if config.mode != inner.mode {
-            inner.mode = config.mode;
+        let new_mode = config.mode();
+        if new_mode != inner.mode {
+            inner.mode = new_mode;
 
             // Wake up the maintenance task to replenish the vanguard pools.
             inner.config_tx.maybe_send(|_| config.clone());
@@ -277,7 +241,16 @@ impl<R: Runtime> VanguardMgr<R> {
     /// for selecting [`Layer2`](Layer::Layer2) vanguards.
     /// It will return an error if a [`Layer3`](Layer::Layer3) is requested.
     ///
-    /// Returns an error is vanguards are disabled.
+    /// Returns an error if vanguards are disabled.
+    ///
+    /// Returns a [`NoSuitableRelay`](VanguardMgrError::NoSuitableRelay) error
+    /// if none of our vanguards satisfy the `layer` and `neighbor_exlusion` requirements.
+    ///
+    /// Returns a [`BootstrapRequired`](VanguardMgrError::BootstrapRequired) error
+    /// if called before the vanguard manager has finished bootstrapping,
+    /// or if all the vanguards have become unusable
+    /// (by expiring or no longer being listed in the consensus)
+    /// and we are unable to replenish them.
     ///
     ///  ### Example
     ///
@@ -298,21 +271,15 @@ impl<R: Runtime> VanguardMgr<R> {
 
         let inner = self.inner.read().expect("poisoned lock");
 
-        // TODO HS-VANGUARDS: code smell.
-        //
-        // If select_vanguards() is called before maintain_vanguard_sets() has obtained a netdir
-        // and populated the vanguard sets, this will return a NoSuitableRelay error (because all
-        // our vanguard sets are empty).
-        //
-        // However, in practice, I don't think this can ever happen, because we don't attempt to
-        // build paths until we're done bootstrapping.
-        //
-        // If it turns out this can actually happen in practice, we can work around it by calling
-        // inner.replenish_vanguards(&self.runtime, netdir)? here (using the netdir arg rather than
-        // the one we obtained ourselves), but at that point we might as well abolish the
-        // maintain_vanguard_sets task and do everything synchronously in this function...
+        // All our vanguard sets are empty. This means select_vanguards() was called before
+        // maintain_vanguard_sets() managed to obtain a netdir and populate the vanguard sets,
+        // or all the vanguards have become unusable and we have been unable to replenish them.
+        if inner.vanguard_sets.l2().is_empty() && inner.vanguard_sets.l3().is_empty() {
+            return Err(VanguardMgrError::BootstrapRequired {
+                action: "select vanguard",
+            });
+        }
 
-        // TODO HS-VANGUARDS: come up with something with better UX
         let relay =
             match (layer, inner.mode) {
                 (Layer::Layer2, Full) | (Layer::Layer2, Lite) => inner
@@ -325,13 +292,11 @@ impl<R: Runtime> VanguardMgr<R> {
                         .l3()
                         .pick_relay(rng, netdir, neighbor_exclusion)
                 }
-                // TODO HS-VANGUARDS: perhaps we need a dedicated error variant for this
                 _ => {
-                    return Err(internal!(
-                        "vanguards for layer {layer} are not supported in mode {})",
-                        inner.mode
-                    )
-                    .into());
+                    return Err(VanguardMgrError::LayerNotSupported {
+                        layer,
+                        mode: inner.mode,
+                    });
                 }
             };
 
@@ -586,9 +551,10 @@ mod test {
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
-    use std::fmt;
+    use std::{fmt, time};
 
     use set::TimeBoundVanguard;
+    use tor_config::ExplicitOrAuto;
 
     use super::*;
 
@@ -598,7 +564,7 @@ mod test {
         testnet::{self, construct_custom_netdir_with_params},
         testprovider::TestNetDirProvider,
     };
-    use tor_persist::TestingStateMgr;
+    use tor_persist::{FsStateMgr, TestingStateMgr};
     use tor_rtmock::MockRuntime;
     use Layer::*;
 
@@ -610,6 +576,27 @@ mod test {
     /// Enable full vanguards for hidden services.
     const ENABLE_FULL_VANGUARDS: [(&str, i32); 1] = [("vanguards-hs-service", 2)];
 
+    /// A valid vanguard state file.
+    const VANGUARDS_JSON: &str = include_str!("../testdata/vanguards.json");
+
+    /// A invalid vanguard state file.
+    const INVALID_VANGUARDS_JSON: &str = include_str!("../testdata/vanguards_invalid.json");
+
+    /// Create the `StateMgr`, populating the vanguards.json state file with the specified JSON string.
+    fn state_dir_with_vanguards(vanguards_json: &str) -> (FsStateMgr, tempfile::TempDir) {
+        let dir = tempfile::TempDir::new().unwrap();
+        std::fs::create_dir_all(dir.path().join("state")).unwrap();
+        std::fs::write(dir.path().join("state/vanguards.json"), vanguards_json).unwrap();
+
+        let statemgr = FsStateMgr::from_path_and_mistrust(
+            dir.path(),
+            &fs_mistrust::Mistrust::new_dangerously_trust_everyone(),
+        )
+        .unwrap();
+
+        (statemgr, dir)
+    }
+
     impl fmt::Debug for Vanguard<'_> {
         fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
             f.debug_struct("Vanguard").finish()
@@ -630,7 +617,9 @@ mod test {
 
     /// Create a new VanguardMgr for testing.
     fn new_vanguard_mgr<R: Runtime>(rt: &R, mode: VanguardMode) -> Arc<VanguardMgr<R>> {
-        let config = VanguardConfig { mode };
+        let config = VanguardConfig {
+            mode: ExplicitOrAuto::Explicit(mode),
+        };
         let statemgr = TestingStateMgr::new();
         let lock = statemgr.try_lock().unwrap();
         assert!(lock.held());
@@ -775,12 +764,25 @@ mod test {
             let netdir = testnet::construct_netdir().unwrap_if_sufficient().unwrap();
             let mut rng = testing_rng();
             let exclusion = RelayExclusion::no_relays_excluded();
+            // Wait until the vanguard manager has bootstrapped
+            // (otherwise we'll get a BootstrapRequired error)
+            let _netdir_provider =
+                init_vanguard_sets(rt.clone(), netdir.clone(), Arc::clone(&vanguardmgr)).await;
 
             // Cannot select an L3 vanguard when running in "Lite" mode.
             let err = vanguardmgr
                 .select_vanguard(&mut rng, &netdir, Layer3, &exclusion)
                 .unwrap_err();
-            assert!(matches!(err, VanguardMgrError::Bug(_)), "{err:?}");
+            assert!(
+                matches!(
+                    err,
+                    VanguardMgrError::LayerNotSupported {
+                        layer: Layer::Layer3,
+                        mode: VanguardMode::Lite
+                    }
+                ),
+                "{err}"
+            );
         });
     }
 
@@ -802,7 +804,12 @@ mod test {
                 .unwrap_err();
 
             assert!(
-                matches!(err, VanguardMgrError::NoSuitableRelay(Layer2)),
+                matches!(
+                    err,
+                    VanguardMgrError::BootstrapRequired {
+                        action: "select vanguard"
+                    }
+                ),
                 "{err:?}"
             );
         });
@@ -900,7 +907,11 @@ mod test {
     /// Switch the vanguard "mode" of the VanguardMgr to `mode`,
     /// by calling `VanguardMgr::reconfigure`.
     fn switch_hs_mode_config(vanguardmgr: &VanguardMgr<MockRuntime>, mode: VanguardMode) {
-        let _ = vanguardmgr.reconfigure(&VanguardConfig { mode }).unwrap();
+        let _ = vanguardmgr
+            .reconfigure(&VanguardConfig {
+                mode: ExplicitOrAuto::Explicit(mode),
+            })
+            .unwrap();
 
         assert_eq!(vanguardmgr.mode(), mode);
     }
@@ -1144,4 +1155,85 @@ mod test {
             .await;
         });
     }
+
+    #[test]
+    fn load_from_state_file() {
+        MockRuntime::test_with_various(|rt| async move {
+            // Set the wallclock to a time when some of the stored vanguards are still valid.
+            let now = time::UNIX_EPOCH + Duration::from_secs(1610000000);
+            rt.jump_wallclock(now);
+
+            let config = VanguardConfig {
+                mode: ExplicitOrAuto::Explicit(VanguardMode::Full),
+            };
+
+            // The state file contains no vanguards
+            let (statemgr, _dir) =
+                state_dir_with_vanguards(r#"{ "l2_vanguards": [], "l3_vanguards": [] }"#);
+            let vanguardmgr = VanguardMgr::new(&config, rt.clone(), statemgr, false).unwrap();
+            {
+                let inner = vanguardmgr.inner.read().unwrap();
+
+                // The vanguard sets should be empty too
+                assert!(inner.vanguard_sets.l2().is_empty());
+                assert!(inner.vanguard_sets.l3().is_empty());
+            }
+
+            let (statemgr, _dir) = state_dir_with_vanguards(VANGUARDS_JSON);
+            let vanguardmgr =
+                Arc::new(VanguardMgr::new(&config, rt.clone(), statemgr, false).unwrap());
+            let (initial_l2, initial_l3) = {
+                let inner = vanguardmgr.inner.read().unwrap();
+                let l2_vanguards = inner.vanguard_sets.l2_vanguards().clone();
+                let l3_vanguards = inner.vanguard_sets.l3_vanguards().clone();
+
+                // The sets actually contain 4 and 5 vanguards, respectively,
+                // but the expired ones are discarded.
+                assert_eq!(l2_vanguards.len(), 3);
+                assert_eq!(l3_vanguards.len(), 2);
+                // We don't know how many vanguards we're going to need
+                // until we fetch the consensus.
+                assert_eq!(inner.vanguard_sets.l2_vanguards_target(), 0);
+                assert_eq!(inner.vanguard_sets.l3_vanguards_target(), 0);
+                assert_eq!(inner.vanguard_sets.l2_vanguards_deficit(), 0);
+                assert_eq!(inner.vanguard_sets.l3_vanguards_deficit(), 0);
+
+                (l2_vanguards, l3_vanguards)
+            };
+
+            let netdir = testnet::construct_netdir().unwrap_if_sufficient().unwrap();
+            let _netdir_provider =
+                init_vanguard_sets(rt.clone(), netdir.clone(), Arc::clone(&vanguardmgr)).await;
+            {
+                let inner = vanguardmgr.inner.read().unwrap();
+                let l2_vanguards = inner.vanguard_sets.l2_vanguards();
+                let l3_vanguards = inner.vanguard_sets.l3_vanguards();
+
+                // The sets were replenished with more vanguards
+                assert_eq!(l2_vanguards.len(), 4);
+                assert_eq!(l3_vanguards.len(), 8);
+                // We now know we need 4 L2 vanguards and 8 L3 ones.
+                assert_eq!(inner.vanguard_sets.l2_vanguards_target(), 4);
+                assert_eq!(inner.vanguard_sets.l3_vanguards_target(), 8);
+                assert_eq!(inner.vanguard_sets.l2_vanguards_deficit(), 0);
+                assert_eq!(inner.vanguard_sets.l3_vanguards_deficit(), 0);
+
+                // All of the vanguards read from the state file should still be in the sets.
+                assert!(initial_l2.iter().all(|v| l2_vanguards.contains(v)));
+                assert!(initial_l3.iter().all(|v| l3_vanguards.contains(v)));
+            }
+        });
+    }
+
+    #[test]
+    fn invalid_state_file() {
+        MockRuntime::test_with_various(|rt| async move {
+            let config = VanguardConfig {
+                mode: ExplicitOrAuto::Explicit(VanguardMode::Full),
+            };
+            let (statemgr, _dir) = state_dir_with_vanguards(INVALID_VANGUARDS_JSON);
+            let res = VanguardMgr::new(&config, rt.clone(), statemgr, false);
+            assert!(matches!(res, Err(VanguardMgrError::State(_))));
+        });
+    }
 }
