@@ -498,12 +498,13 @@ impl RouterDesc {
         };
 
         // master-key-ed25519: required, and should match certificate.
+        #[allow(unexpected_cfgs)]
         {
             let master_key_tok = body.required(MASTER_KEY_ED25519)?;
             let ed_id: Ed25519Public = master_key_tok.parse_arg(0)?;
             let ed_id: ll::pk::ed25519::Ed25519Identity = ed_id.into();
             if ed_id != *identity_cert.peek_signing_key() {
-                #[cfg(not(fuzzing))]
+                #[cfg(not(fuzzing))] // No feature here; never omit in production.
                 return Err(EK::BadObjectVal
                     .at_pos(master_key_tok.pos())
                     .with_msg("master-key-ed25519 does not match key in identity-ed25519"));
