@@ -37,6 +37,7 @@ use crate::crypto::handshake::fast::CreateFastClient;
 use crate::crypto::handshake::ntor_v3::{NtorV3Client, NtorV3PublicKey};
 use crate::stream::{AnyCmdChecker, StreamStatus};
 use crate::util::err::{ChannelClosed, ReactorError};
+use crate::util::SinkExt as _;
 use crate::{Error, Result};
 use std::borrow::Borrow;
 use std::collections::VecDeque;
@@ -63,7 +64,7 @@ use tor_error::internal;
 use std::sync::{Arc, Mutex};
 use std::task::{Context, Poll};
 
-use crate::channel::Channel;
+use crate::channel::{Channel, ChannelSender};
 use crate::circuit::path;
 #[cfg(test)]
 use crate::circuit::sendme::CircTag;
@@ -707,8 +708,10 @@ pub struct Reactor {
     /// NOTE: Control messages could potentially add unboundedly to this, although that's
     ///       not likely to happen (and isn't triggereable from the network, either).
     outbound: VecDeque<AnyChanCell>,
-    /// The channel this circuit is using to send cells through.
-    channel: Channel,
+    /// The channel this circuit is attached to.
+    channel: Arc<Channel>,
+    /// Sender object used to actually send cells.
+    chan_sender: ChannelSender,
     /// A oneshot sender that is used to alert other tasks when this reactor is
     /// finally dropped.
     ///
@@ -790,7 +793,7 @@ impl Reactor {
     ///
     /// The internal unique identifier for this circuit will be `unique_id`.
     pub(super) fn new(
-        channel: Channel,
+        channel: Arc<Channel>,
         channel_id: CircId,
         unique_id: UniqId,
         input: mpsc::Receiver<ClientCircChanMsg>,
@@ -808,11 +811,14 @@ impl Reactor {
 
         let (reactor_closed_tx, reactor_closed_rx) = oneshot::channel();
 
+        let chan_sender = channel.sender();
+
         let reactor = Reactor {
             control: control_rx,
             reactor_closed_tx,
             outbound: Default::default(),
             channel,
+            chan_sender,
             input,
             crypto_in: InboundClientCrypt::new(),
             hops: vec![],
@@ -903,106 +909,108 @@ impl Reactor {
             let mut streams_to_close = vec![];
             let mut stream_relaycells = vec![];
 
-            // Is the channel ready to receive anything at all?
-            if self.channel.poll_ready(cx)? {
-                // (using this as a named block for early returns; not actually a loop)
-                #[allow(clippy::never_loop)]
-                'outer: loop {
-                    // First, drain our queue of things we tried to send earlier, but couldn't.
-                    while let Some(msg) = self.outbound.pop_front() {
-                        trace!("{}: sending from enqueued: {:?}", self.unique_id, msg);
-                        Pin::new(&mut self.channel).start_send(msg)?;
+            // (using this as a named block for early returns; not actually a loop)
+            #[allow(clippy::never_loop)]
+            'outer: loop {
+                // First, drain our queue of things we tried to send earlier, but couldn't.
+                loop {
+                    // `futures::Sink::start_send` dictates we need to call `poll_ready` before
+                    // each `start_send` call.
+                    if !self.chan_sender.poll_ready_unpin_bool(cx)? {
+                        break 'outer;
+                    }
+                    let Some(msg) = self.outbound.pop_front() else {
+                        break;
+                    };
+                    trace!("{}: sending from enqueued: {:?}", self.unique_id, msg);
+                    Pin::new(&mut self.chan_sender).start_send(msg)?;
+                }
 
-                        // `futures::Sink::start_send` dictates we need to call `poll_ready` before
-                        // each `start_send` call.
-                        if !self.channel.poll_ready(cx)? {
+                // Let's look at our hops, and streams for each hop.
+                for i in 0..self.hops.len() {
+                    let hop_num = HopNum::from(i as u8);
+                    // If we can, drain our queue of things we tried to send earlier, but
+                    // couldn't due to congestion control.
+                    'hop_outbound: loop {
+                        if !self.chan_sender.poll_ready_unpin_bool(cx)? {
                             break 'outer;
                         }
-                    }
-
-                    // Let's look at our hops, and streams for each hop.
-                    for i in 0..self.hops.len() {
-                        let hop_num = HopNum::from(i as u8);
-                        // If we can, drain our queue of things we tried to send earlier, but
-                        // couldn't due to congestion control.
-                        if self.hops[i].sendwindow.window() > 0 {
-                            'hop: while let Some((early, cell)) = self.hops[i].outbound.pop_front()
-                            {
-                                trace!(
-                                    "{}: sending from hop-{}-enqueued: {:?}",
-                                    self.unique_id,
-                                    i,
-                                    cell
-                                );
-                                // We know the stream has a non-empty SENDME
-                                // window because we accept at most one cell per
-                                // stream below, and only when the relevant
-                                // stream has a non-empty window and after
-                                // clearing the backlog here.
-                                //
-                                // TODO prop340: We need to be careful here when
-                                // adding fragmentation; e.g. this argument
-                                // fails to hold if we allow later breaking a
-                                // DATA message into multiple messages and cells
-                                // for packing.
-                                self.send_relay_cell(cx, hop_num, early, cell)?;
-                                if !self.channel.poll_ready(cx)? {
-                                    break 'outer;
-                                }
-                                if self.hops[i].sendwindow.window() == 0 {
-                                    break 'hop;
-                                }
-                            }
+                        if self.hops[i].sendwindow.window() == 0 {
+                            break 'hop_outbound;
                         }
-                        let hop = &mut self.hops[i];
-                        // Look at all of the streams on this hop.
-                        for (id, stream) in hop.map.iter_mut() {
-                            if let StreamEntMut::Open(OpenStreamEnt {
-                                rx, send_window, ..
-                            }) = stream
-                            {
-                                // Do the stream and hop send windows allow us to obtain and
-                                // send something?
-                                //
-                                // NOTE: not everything counts toward congestion
-                                // control. However, we can't easily remove
-                                // this check:
-                                // * We need to be careful not to buffer more
-                                // than ONE message per stream for the call to
-                                // `send_relay_cell` above, and potentially
-                                // closing the stream below, to be correct.
-                                // * We need to be careful about allowing
-                                // messages that *don't* count to be sent before
-                                // messages that *do*; e.g. we wouldn't want to
-                                // accept and send an END message on a stream where we
-                                // still have DATA messages queued.
-                                if send_window.window() > 0 && hop.sendwindow.window() > 0 {
-                                    match Pin::new(rx).poll_next(cx) {
-                                        Poll::Ready(Some(m)) => {
-                                            stream_relaycells.push((
-                                                hop_num,
-                                                AnyRelayMsgOuter::new(Some(id), m),
-                                            ));
-                                        }
-                                        Poll::Ready(None) => {
-                                            // Stream receiver was dropped; close the stream.
-                                            //
-                                            // We know there are no queued messages for the stream
-                                            // since we already flushed above.
-                                            //
-                                            // We can't close it here due to
-                                            // borrowck; that will happen later.
-                                            streams_to_close.push((hop_num, id));
-                                        }
-                                        Poll::Pending => {}
-                                    }
+                        let Some((early, cell)) = self.hops[i].outbound.pop_front() else {
+                            break 'hop_outbound;
+                        };
+                        trace!(
+                            "{}: sending from hop-{}-enqueued: {:?}",
+                            self.unique_id,
+                            i,
+                            cell
+                        );
+                        // We know the stream has a non-empty SENDME
+                        // window because we accept at most one cell per
+                        // stream below, and only when the relevant
+                        // stream has a non-empty window and after
+                        // clearing the backlog here.
+                        //
+                        // TODO prop340: We need to be careful here when
+                        // adding fragmentation; e.g. this argument
+                        // fails to hold if we allow later breaking a
+                        // DATA message into multiple messages and cells
+                        // for packing.
+                        self.send_relay_cell(cx, hop_num, early, cell)?;
+                    }
+                    let hop = &mut self.hops[i];
+                    // Look at all of the streams on this hop.
+                    for (id, stream) in hop.map.iter_mut() {
+                        let StreamEntMut::Open(OpenStreamEnt {
+                            rx, send_window, ..
+                        }) = stream
+                        else {
+                            continue;
+                        };
+                        // Do the stream and hop send windows allow us to obtain and
+                        // send something?
+                        //
+                        // NOTE: not everything counts toward congestion
+                        // control. However, we can't easily remove
+                        // this check:
+                        // * We need to be careful not to buffer more
+                        // than ONE message per stream for the call to
+                        // `send_relay_cell` above, and potentially
+                        // closing the stream below, to be correct.
+                        // * We need to be careful about allowing
+                        // messages that *don't* count to be sent before
+                        // messages that *do*; e.g. we wouldn't want to
+                        // accept and send an END message on a stream where we
+                        // still have DATA messages queued.
+                        if send_window.window() > 0 && hop.sendwindow.window() > 0 {
+                            match Pin::new(rx).poll_next(cx) {
+                                Poll::Ready(Some(m)) => {
+                                    stream_relaycells
+                                        .push((hop_num, AnyRelayMsgOuter::new(Some(id), m)));
                                 }
+                                Poll::Ready(None) => {
+                                    // Stream channel was closed, so we'll close the stream.
+                                    //
+                                    // This can happen either when the last StreamTarget for the
+                                    // stream is dropped, or when StreamTarget::close() is called
+                                    // on one of the stream targets.
+                                    //
+                                    // We know there are no queued messages for the stream
+                                    // since we already flushed those above.
+                                    //
+                                    // We can't close it here due to
+                                    // borrowck; that will happen later.
+                                    streams_to_close.push((hop_num, id));
+                                }
+                                Poll::Pending => {}
                             }
                         }
                     }
-
-                    break;
                 }
+
+                break;
             }
 
             // Close the streams we said we'd close.
@@ -1022,7 +1030,7 @@ impl Reactor {
                 did_things = true;
             }
 
-            let _ = Pin::new(&mut self.channel)
+            let _ = Pin::new(&mut self.chan_sender)
                 .poll_flush(cx)
                 .map_err(|_| ChannelClosed)?;
 
@@ -1102,7 +1110,7 @@ impl Reactor {
         let _ = done.send(ret); // don't care if sender goes away
 
         futures::future::poll_fn(|cx| -> Poll<Result<()>> {
-            let _ = Pin::new(&mut self.channel)
+            let _ = Pin::new(&mut self.chan_sender)
                 .poll_flush(cx)
                 .map_err(|_| ChannelClosed)?;
             Poll::Ready(Ok(()))
@@ -1466,8 +1474,8 @@ impl Reactor {
         //            cells to be sent out of order, since it could transition from not ready to
         //            ready during one cycle of the reactor!
         //            (This manifests as a protocol violation.)
-        if self.outbound.is_empty() && self.channel.poll_ready(cx)? {
-            Pin::new(&mut self.channel).start_send(cell)?;
+        if self.outbound.is_empty() && self.chan_sender.poll_ready_unpin_bool(cx)? {
+            Pin::new(&mut self.chan_sender).start_send(cell)?;
         } else {
             // This has been observed to happen in code that doesn't have bugs in it, simply due
             // to the way `Channel`'s `poll_ready` implementation works (it can change due to
@@ -1962,7 +1970,7 @@ impl Reactor {
                     ))
                 })?
                 .recvwindow
-                .put();
+                .put()?;
         }
 
         let (mut msgs, incomplete) = decode_res.into_parts();
