@@ -4,10 +4,10 @@ use bitvec::prelude::*;
 
 use super::{AnyCmdChecker, DataStream, StreamReader, StreamStatus};
 use crate::circuit::reactor::CloseStreamBehavior;
-use crate::circuit::StreamTarget;
+use crate::circuit::{ClientCircSyncView, StreamTarget};
 use crate::{Error, Result};
 use tor_async_utils::oneshot;
-use tor_cell::relaycell::{msg, RelayCmd, UnparsedRelayCell};
+use tor_cell::relaycell::{msg, RelayCmd, UnparsedRelayMsg};
 use tor_cell::restricted_msg;
 use tor_error::internal;
 
@@ -151,7 +151,7 @@ impl IncomingCmdChecker {
 }
 
 impl super::CmdChecker for IncomingCmdChecker {
-    fn check_msg(&mut self, msg: &UnparsedRelayCell) -> Result<StreamStatus> {
+    fn check_msg(&mut self, msg: &UnparsedRelayMsg) -> Result<StreamStatus> {
         if self.allow_commands[u8::from(msg.cmd()) as usize] {
             Ok(StreamStatus::Open)
         } else {
@@ -162,7 +162,7 @@ impl super::CmdChecker for IncomingCmdChecker {
         }
     }
 
-    fn consume_checked_msg(&mut self, msg: UnparsedRelayCell) -> Result<()> {
+    fn consume_checked_msg(&mut self, msg: UnparsedRelayMsg) -> Result<()> {
         let _ = msg
             .decode::<IncomingStreamRequest>()
             .map_err(|err| Error::from_bytes_err(err, "invalid message on incoming stream"))?;
@@ -171,12 +171,56 @@ impl super::CmdChecker for IncomingCmdChecker {
     }
 }
 
+/// A callback that can check whether a given stream request is acceptable
+/// immediately on its receipt.
+///
+/// This should only be used for checks that need to be done immediately, with a
+/// view of the state of the circuit.  Any other checks should, if possible, be
+/// done on the [`IncomingStream`] objects as they are received.
+pub trait IncomingStreamRequestFilter: Send + 'static {
+    /// Check an incoming stream request, and decide what to do with it.
+    ///
+    /// Implementations of this function should
+    fn disposition(
+        &mut self,
+        ctx: &IncomingStreamRequestContext<'_>,
+        circ: &ClientCircSyncView<'_>,
+    ) -> Result<IncomingStreamRequestDisposition>;
+}
+
+/// What action to take with an incoming stream request.
+#[derive(Clone, Debug)]
+#[non_exhaustive]
+pub enum IncomingStreamRequestDisposition {
+    /// Accept the request (for now) and pass it to the mpsc::Receiver
+    /// that is yielding them as [`IncomingStream``
+    Accept,
+    /// Rejected the request, and close the circuit on which it was received.
+    CloseCircuit,
+    /// Reject the request and send an END message.
+    RejectRequest(msg::End),
+}
+
+/// Information about a stream request, as passed to an [`IncomingStreamRequestFilter`].
+pub struct IncomingStreamRequestContext<'a> {
+    /// The request message itself
+    pub(crate) request: &'a IncomingStreamRequest,
+}
+
+impl<'a> IncomingStreamRequestContext<'a> {
+    /// Return a reference to the message used to request this stream.
+    pub fn request(&self) -> &'a IncomingStreamRequest {
+        self.request
+    }
+}
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
@@ -188,7 +232,7 @@ mod test {
 
     use tor_cell::relaycell::{
         msg::{Begin, BeginDir, Data, Resolve},
-        AnyRelayMsgOuter,
+        AnyRelayMsgOuter, RelayCellFormat,
     };
 
     use super::*;
@@ -200,7 +244,7 @@ mod test {
             let body = AnyRelayMsgOuter::new(None, msg)
                 .encode(&mut rand::thread_rng())
                 .unwrap();
-            UnparsedRelayCell::from_body(body)
+            UnparsedRelayMsg::from_singleton_body(RelayCellFormat::V0, body).unwrap()
         };
         let begin = u(Begin::new("allium.example.com", 443, 0).unwrap().into());
         let begin_dir = u(BeginDir::default().into());
