@@ -1,7 +1,7 @@
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
-#![cfg_attr(not(ci_arti_stable), allow(renamed_and_removed_lints))]
-#![cfg_attr(not(ci_arti_nightly), allow(unknown_lints))]
+#![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
+#![allow(unknown_lints)] // @@REMOVE_WHEN(ci_arti_nightly)
 #![warn(missing_docs)]
 #![warn(noop_method_call)]
 #![warn(unreachable_pub)]
@@ -49,8 +49,11 @@ use std::{convert::Infallible, sync::Arc};
 
 pub use dispatch::{DispatchTable, InvokeError, UpdateSink};
 pub use err::RpcError;
-pub use method::{is_method_name, iter_method_names, DynMethod, Method, NoUpdates};
-pub use obj::{Object, ObjectId, ObjectRefExt};
+pub use method::{
+    check_method_names, is_method_name, iter_method_names, DeserMethod, DynMethod,
+    InvalidMethodName, Method, NoUpdates,
+};
+pub use obj::{Object, ObjectArcExt, ObjectId};
 
 #[doc(hidden)]
 pub use obj::cast::CastTable;
@@ -116,6 +119,9 @@ pub trait Context: Send {
     ///
     /// TODO RPC should this really return a LookupError?
     fn release_owned(&self, object: &ObjectId) -> Result<(), LookupError>;
+
+    /// Return a dispatch table that can be used to invoke other RPC methods.
+    fn dispatch_table(&self) -> &Arc<std::sync::RwLock<DispatchTable>>;
 }
 
 /// An error caused while trying to send an update to a method.
@@ -172,3 +178,13 @@ impl<T: Context> ContextExt for T {}
 #[derive(Debug, Clone, Copy, serde::Serialize, serde::Deserialize, Default)]
 #[non_exhaustive]
 pub struct Nil {}
+/// An instance of rpc::Nil.
+pub const NIL: Nil = Nil {};
+
+/// Common return type for RPC methods that return a single object ID
+/// and nothing else.
+#[derive(Debug, Clone, serde::Serialize, serde::Deserialize, derive_more::From)]
+pub struct SingletonId {
+    /// The ID of the object that we're returning.
+    id: ObjectId,
+}
